/* Copyright (C) 2021 Goethe-University Frankfurt, Frankfurt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Pascal Raisig, Dominik Smith [committer] */

#include "UnpackRich.h"

#include <cstdint>

#include "AlgoFairloggerCompat.h"

namespace cbm::algo
{
  // ----   Algorithm execution   ---------------------------------------------
  UnpackRich::resultType UnpackRich::operator()(const uint8_t* msContent, const fles::MicrosliceDescriptor& msDescr,
                                                const uint64_t tTimeslice)
  {
    // --- Output data
    resultType result = {};

    fMonitor = UnpackRichMonitorData();
    fOutputVec.clear();

    // Clear CbmTime of MS. Used to get time offset of subtriggers to MS start
    fCbmTimeMS = 0;

    RichMicrosliceReader reader;
    reader.SetData(msContent, msDescr.size);

    const auto mstime = msDescr.idx;
    fMsRefTime        = mstime - tTimeslice;

    // There are a lot of MS  with 8 bytes size
    // Does one need these MS?
    if (reader.GetSize() <= 8) return result;

    while (true) {
      ProcessTrbPacket(reader);
      ProcessHubBlock(reader);

      // -4*2 for 2 last words which contain microslice index
      if (reader.GetOffset() >= reader.GetSize() - 8) break;
      // -4*3 for 0xffffffff padding and 2 last words which contain microslice index
      if (reader.IsNextPadding() && reader.GetOffset() >= reader.GetSize() - 12) break;
    }
    uint32_t msIndexWord1 = reader.NextWord();
    uint32_t msIndexWord2 = reader.NextWord();

    result.first  = fOutputVec;
    result.second = fMonitor;
    return result;
  }


  void UnpackRich::ProcessTrbPacket(RichMicrosliceReader& reader)
  {
    //process CBM time
    const uint32_t word_MSB = reader.NextWord();  // CBM 63:32
    const uint32_t word_LSB = reader.NextWord();  // CBM 31: 0
    fCbmTimePacket          = (uint64_t) word_MSB << 32 | word_LSB;
    if (fCbmTimeMS == 0) fCbmTimeMS = fCbmTimePacket;

    //discard unused words
    for (auto l = 0; l < 10; ++l) {
      reader.NextWord();
    }

    //process remaining words
    for (auto l = 0; l < 14; ++l) {
      const uint32_t wordEpoch = reader.NextWord();
      const uint32_t epoch     = ProcessEpoch(wordEpoch);
      const uint32_t wordTime  = reader.NextWord();
      const RichTdcTimeData td = ProcessTimeData(wordTime);
      const double fullTime    = CalculateTime(epoch, td.fCoarse, td.fFine);

      if (l == 0) fPrevLastCh0ReTime[12] = fullTime;
      if (l == 1) fMbsCorr = fullTime - fPrevLastCh0ReTime[12];  // = MbsPrevTimeCh1 - MbsPrevTimeCh0
      if (l > 1) fPrevLastCh0ReTime[l - 2] = fullTime;
    }

    const uint32_t trbNum = reader.NextWord();  // TRB trigger number
    //if (isLog()) L_(debug4) << getLogHeader(reader) << "TRB Num:" << reader.GetWordAsHexString(trbNum);
  }


  void UnpackRich::ProcessHubBlock(RichMicrosliceReader& reader)
  {
    uint32_t word          = reader.NextWord();
    const uint32_t hubId   = word & 0xffff;          // 16 bits
    const uint32_t hubSize = (word >> 16) & 0xffff;  // 16 bits

    bool isLast         = false;  // if true then it is CTS sub-sub-event
    size_t totalSize    = 0;
    fCurrentSubSubEvent = 0;

    uint32_t subSubEventId, subSubEventSize;

    //loop over events in hub block
    while (totalSize < hubSize) {
      word            = reader.NextWord();
      subSubEventId   = word & 0xffff;                              // 16 bits
      subSubEventSize = (word >> 16) & 0xffff;                      // 16 bits
      isLast          = reader.IsLastSubSubEvent(subSubEventSize);  // if true then it is CTS sub-sub-event
      totalSize += (1 + subSubEventSize);

      if (!isLast) {                                 // all except last are DiRICH events
        if (((subSubEventId >> 12) & 0xF) != 0x7) {  // catch invalid ids
          fMonitor.fNumErrInvalidHubId++;
        }
        if (totalSize == hubSize) { fMonitor.fNumErrInvalidHubSize++; }
        ProcessSubSubEvent(reader, subSubEventSize, subSubEventId);
        fCurrentSubSubEvent++;
      }
    }

    //last event ist expected to be CTS
    if (totalSize != hubSize || !isLast) { fMonitor.fNumErrInvalidHubSize++; }
    subSubEventSize = ProcessCtsHeader(reader, subSubEventSize, subSubEventId);
    ProcessSubSubEvent(reader, subSubEventSize, subSubEventId);

    // read last words
    int lastWordsCounter = 0;
    while (true) {
      lastWordsCounter++;
      word = reader.NextWord();
      if (word == 0x600dda7a) {
        if (reader.IsNextPadding()) word = reader.NextWord();
        break;
      }
      if (lastWordsCounter >= 7) { fMonitor.fNumErrExcessLastWords++; }
    }
  }

  int UnpackRich::ProcessCtsHeader(RichMicrosliceReader& reader, uint32_t subSubEventSize, uint32_t subSubEventId)
  {
    const uint32_t word         = reader.NextWord();
    const uint32_t ctsState     = word & 0xffff;                                                            // 16 bits
    const uint32_t nofInputs    = (word >> 16) & 0xf;                                                       // 4 bits
    const uint32_t nofTrigCh    = (word >> 20) & 0x1f;                                                      // 5 bits
    const uint32_t inclLastIdle = (word >> 25) & 0x1;                                                       // 1 bit
    const uint32_t inclTrigInfo = (word >> 26) & 0x1;                                                       // 1 bit
    const uint32_t inclTime     = (word >> 27) & 0x1;                                                       // 1 bit
    const uint32_t ETM          = (word >> 28) & 0x3;                                                       // 2 bits
    uint32_t ctsInfoSize = 2 * nofInputs + 2 * nofTrigCh + 2 * inclLastIdle + 3 * inclTrigInfo + inclTime;  // in words
    switch (ETM) {
      case 0: break;
      case 1: ctsInfoSize += 1; break;
      case 2: ctsInfoSize += 4; break;
      case 3: break;
    }
    for (uint32_t i = 0; i < ctsInfoSize; i++) {
      reader.NextWord();  // do nothing?
    }
    const int nofTimeWords = subSubEventSize - ctsInfoSize - 1;
    return nofTimeWords;
  }

  void UnpackRich::ProcessSubSubEvent(RichMicrosliceReader& reader, int nofTimeWords, uint32_t subSubEventId)
  {
    uint32_t epoch = 0;  // store last epoch obtained in sub-sub-event

    // Store last raising edge time for each channel or -1. if no time
    // this array is used to match raising and falling edges
    std::vector<double> raisingTime(33, -1.);

    // check if DiRICH (SubSubEvId) is masked and skip to end if so
    if (CheckMaskedDiRICH(subSubEventId)) {
      for (int i = 0; i < nofTimeWords; i++) {
        reader.NextWord();
      }
      return;
    }

    // First word is expected to be of type "header"
    if (GetTdcWordType(reader.NextWord()) != RichTdcWordType::Header) { fMonitor.fNumErrInvalidFirstMessage++; }

    // Second word is expected to be of type "epoch"
    uint32_t word = reader.NextWord();
    if (GetTdcWordType(word) != RichTdcWordType::Epoch) { fMonitor.fNumErrInvalidSecondMessage++; }
    else {
      epoch = ProcessEpoch(word);
    }

    // Loop over words
    for (int i = 2; i < nofTimeWords - 1; i++) {
      word = reader.NextWord();
      switch (GetTdcWordType(word)) {
        case RichTdcWordType::TimeData: {
          ProcessTimeDataWord(epoch, word, subSubEventId, raisingTime);
          break;
        }
        case RichTdcWordType::Epoch: {
          epoch = ProcessEpoch(word);
          break;
        }
        case RichTdcWordType::Header: {
          fMonitor.fNumErrWildHeaderMessage++;
          break;
        }
        case RichTdcWordType::Trailer: {
          fMonitor.fNumErrWildTrailerMessage++;
          break;
        }
        case RichTdcWordType::Debug: {
          break;
          // for the moment do nothing
          fMonitor.fNumDebugMessage++;
          break;
        }
        case RichTdcWordType::Error: {
          fMonitor.fNumErrTdcErrorWord++;
          break;
        }
      }
    }

    // Last word is expected to be of type "trailer"
    if (GetTdcWordType(reader.NextWord()) != RichTdcWordType::Trailer) { fMonitor.fNumErrInvalidLastMessage++; }
  }

  // ---- ProcessTimeDataWord ----
  void UnpackRich::ProcessTimeDataWord(uint32_t epoch, uint32_t tdcWord, uint32_t subSubEventId,
                                       std::vector<double>& raisingTime)
  {
    const RichTdcTimeData td = ProcessTimeData(tdcWord);
    const double fullTime    = CalculateTime(epoch, td.fCoarse, td.fFine);

    if (td.fChannel != 0) {
      const double dT            = fullTime - fPrevLastCh0ReTime[fCurrentSubSubEvent];
      const double subtrigOffset = (fCbmTimePacket - fCbmTimeMS) * 25.0;  // offset of SubTrigger to MS start in ns
      const double fullTimeCorr  = dT - fMbsCorr + subtrigOffset;

      if (td.fChannel < 1 || td.fChannel >= raisingTime.size()) { fMonitor.fNumErrChannelOutOfBounds++; }
      if (td.fIsRisingEdge) {
        // always store the latest raising edge. It means that in case RRFF situation only middle RF will be matched.
        raisingTime[td.fChannel] = fullTimeCorr;
      }
      else {
        if (raisingTime[td.fChannel] != -1.) {
          // Matching was found, calculate ToT, if tot is in a good range -> create digi
          const double ToT = fullTimeCorr - raisingTime[td.fChannel];
          if (ToT >= fToTMin && ToT <= fToTMax) {
            if (fullTimeCorr >= 0.0) { WriteOutputDigi(subSubEventId, td.fChannel, raisingTime[td.fChannel], ToT); }
          }
          // pair was created, set raising edge to -1.
          raisingTime[td.fChannel] = -1.;
        }
      }
    }
  }

  double UnpackRich::CalculateTime(uint32_t epoch, uint32_t coarse, uint32_t fine)
  {
    return ((double) epoch) * 2048. * 5. + ((double) coarse) * 5. - ((double) fine) * 0.005;
  }

  void UnpackRich::WriteOutputDigi(int32_t fpgaID, int32_t channel, double time, double tot)
  {
    double ToTcorr   = fbDoToTCorr ? fParams.fElinkParams[fpgaID].fToTshift[channel] : 0.;
    int32_t pixelUID = GetPixelUID(fpgaID, channel);
    //check ordering
    double finalTime = time + (double) fMsRefTime - fParams.fElinkParams[fpgaID].fTimeOffset;

    // Do not accept digis, where the MS und TS differs by more than 6 sec (mainly TS0)
    if (6e9 < finalTime) return;

    fOutputVec.emplace_back(pixelUID, finalTime, tot - ToTcorr);
  }

  bool UnpackRich::CheckMaskedDiRICH(int32_t subSubEventId)
  {
    for (unsigned int i = 0; i < fMaskedDiRICHes.size(); ++i) {
      if (fMaskedDiRICHes.at(i) == subSubEventId) return true;
    }
    return false;
  }

  RichTdcWordType UnpackRich::GetTdcWordType(uint32_t tdcWord)
  {
    uint32_t tdcTimeDataMarker = (tdcWord >> 31) & 0x1;  // 1 bit
    uint32_t tdcMarker         = (tdcWord >> 29) & 0x7;  // 3 bits

    if (tdcTimeDataMarker == 0x1) {
      // TODO: I also include tdcMarker == 0x5, some tdc time data words have this marker. Is it correct?
      if (tdcMarker == 0x4 || tdcMarker == 0x5) { return RichTdcWordType::TimeData; }
      else {
        return RichTdcWordType::Error;
      }
    }
    if (tdcMarker == 0x0) return RichTdcWordType::Trailer;
    if (tdcMarker == 0x1) return RichTdcWordType::Header;
    if (tdcMarker == 0x2) return RichTdcWordType::Debug;
    if (tdcMarker == 0x3) return RichTdcWordType::Epoch;
    return RichTdcWordType::Error;
  }

  int32_t UnpackRich::GetPixelUID(int32_t fpgaID, int32_t ch)
  {
    // First 16 bits are used for the FPGA ID, then
    // 8 bits unused and then 8 bits are used for the channel
    return ((fpgaID << 16) | (ch & 0x00FF));
  }

  RichTdcTimeData UnpackRich::ProcessTimeData(uint32_t tdcWord)
  {
    RichTdcTimeData out;
    out.fCoarse       = static_cast<uint32_t>(tdcWord & 0x7ff);          // 11 bits
    out.fIsRisingEdge = static_cast<uint32_t>((tdcWord >> 11) & 0x1);    // 1 bit
    out.fFine         = static_cast<uint32_t>((tdcWord >> 12) & 0x3ff);  // 10 bits
    out.fChannel      = static_cast<uint32_t>((tdcWord >> 22) & 0x7f);   // 7 bits
    return out;
  }

  uint32_t UnpackRich::ProcessEpoch(uint32_t tdcWord) { return static_cast<uint32_t>(tdcWord & 0xfffffff); }

  uint16_t UnpackRich::ProcessHeader(uint32_t tdcWord)
  {
    return static_cast<uint16_t>(tdcWord & 0xff);  //8 bits
  }

  uint16_t UnpackRich::ProcessTrailer(uint32_t tdcWord) { return static_cast<uint16_t>(tdcWord & 0xffff); }

} /* namespace cbm::algo */
